@@ -41,11 +41,17 @@
 #' interface. For the most part, the two functions can be used interchangeably by
 #' users.
 #' 
-#' @param x An object defining how to create the distances
+#' @param x An object defining how to create the distances. All methods require
+#' some form of names (e.g. \code{names} for vectors or \code{rownames} for
+#' matrix like objects)
 #' @param within A valid distance specification, such as the result
 #' of \code{\link{exactMatch}} or \code{\link{caliper}}. Finite entries indicate
 #' which distances to create. Including this argument can significantly speed up
 #' computation for sparse matching problems.
+#' @param caliper The width of a caliper to use to exclude treated-control
+#' pairs with values greater than the width. For some methods, there may be a
+#' speed advantage to passing a width rather than using the
+#' \code{\link{caliper}} function on an existing distance specification.
 #' @return A distance specification (a matrix or similar object) which is
 #' suitable to be given as the \code{distance} argument to \code{\link{fullmatch}}
 #' or \code{\link{pairmatch}}. 
@@ -61,7 +67,7 @@
 #' @docType methods
 #' @rdname match_on-methods
 #' @aliases InfinitySparseMatrix-class
-setGeneric("match_on", def = function(x, within = NULL, ...) {
+setGeneric("match_on", def = function(x, within = NULL, caliper = NULL, ...) {
 
   tmp <- standardGeneric("match_on")
   tmp@call <- match.call()
@@ -79,17 +85,17 @@ setGeneric("match_on", def = function(x, within = NULL, ...) {
 #' control unit, again in the \code{data} object. For each of these pairs, the
 #' function should return the distance between the treated unit and control unit.
 #' This may sound complicated, but is simple to use. For example, a function that
-#' returned the absolute difference between to units using a vector of data would
+#' returned the absolute difference between two units using a vector of data would
 #' be 
 #' \code{f <- function(index, data, z) { abs(apply(index, 1, function(pair) { data[pair[1]] - data[pair[2]] })) }}.
 #' (Note: This simple case is precisely handled by the \code{numeric} method.)
 #' 
 #' @param z A factor, logical, or binary vector indicating treatment (the higher level) and control (the lower level) for each unit in the study.
 #' @param data A \code{data.frame} or \code{matrix} containing variables used by the method to construct the distance matrix.
-#' @usage \S4method{match_on}{function}(x, within = NULL, z = NULL, data = NULL, ...)
+#' @usage \S4method{match_on}{function}(x, within = NULL, caliper = NULL, z = NULL, data = NULL, ...)
 #' @rdname match_on-methods
 #' @aliases match_on,function-method
-setMethod("match_on", "function", function(x, within = NULL, z = NULL, data = NULL, ...) {
+setMethod("match_on", "function", function(x, within = NULL, caliper = NULL, z = NULL, data = NULL, ...) {
 
   if (is.null(data) | is.null(z)) {
     stop("Data and treatment indicator arguments are required.")
@@ -97,22 +103,38 @@ setMethod("match_on", "function", function(x, within = NULL, z = NULL, data = NU
 
   theFun <- match.fun(x)
 
-  makedist(z, data, theFun, within)
+  tmp <- makedist(z, data, theFun, within)
+  
+  if (is.null(caliper)) {
+    return(tmp)  
+  }
+
+  return(tmp + optmatch::caliper(tmp, width = caliper))
 })
 
 
 #' @details The formula method produces, by default, a Mahalanobis distance specification
 #' based on the formula \code{Z ~ X1 + X2 + ... }, where
-#' \code{Z}  the treatment indicator. A Mahalanobis
-#' distance scales the Euclidean distance by the inverse of the
-#' covariance matrix. Other options can be selected by the \code{method} argument. 
+#' \code{Z} is the treatment indicator. The Mahalanobis distance is calculated as the 
+#' square root of d'Cd, where d is the vector of X-differences on a pair of observations and C
+#' is an inverse (generalized inverse) of the pooled covariance of Xes. (The pooling is of the
+#' covariance of X within the subset defined by \code{Z==0} and within the complement of that 
+#' subset. This is similar to a Euclidean distance calculated after reexpressing the Xes in 
+#' standard units, such that the reexpressed variables all have pooled SDs of 1; except that 
+#' it addresses redundancies among the variables by scaling down variables contributions in 
+#' proportion to their correlations with other included variables.) 
+#' 
+#' Euclidean distance is also available, via \code{method="euclidean"}. Or, implement your own;  
+#' for hints as to how, refer to\cr
+#' \url{https://github.com/markmfredrickson/optmatch/wiki/How-to-write-your-own-compute-method}
+#' 
 #' @param subset A subset of the data to use in creating the distance specification.
 #' @param method A string indicating which method to use in computing the distances from the data. 
 #' The current possibilities are \code{"mahalanobis", "euclidean"}. 
-#' @usage \S4method{match_on}{formula}(x, within = NULL, data = NULL, subset = NULL, method = "mahalanobis", ...)
+#' @usage \S4method{match_on}{formula}(x, within = NULL, caliper = NULL, data = NULL, subset = NULL, method = "mahalanobis", ...)
 #' @rdname match_on-methods
 #' @aliases match_on,formula-method
-setMethod("match_on", "formula", function(x, within = NULL, data = NULL, subset = NULL, 
+setMethod("match_on", "formula", function(x, within = NULL, caliper = NULL, data = NULL, subset = NULL, 
                                        method = "mahalanobis", ...) {
   if (length(x) != 3) {
     stop("Formula must have a left hand side.")  
@@ -137,34 +159,49 @@ setMethod("match_on", "formula", function(x, within = NULL, data = NULL, subset
   names(z) <- rownames(mf)
  
   f <- match.fun(paste("compute_", method, sep = ""))
-  makedist(z, data, f, within)
 
+  tmp <- makedist(z, data, f, within)
+
+  if (is.null(caliper)) {
+    return(tmp)  
+  }
+
+  return(tmp + optmatch::caliper(tmp, width = caliper))
 })
 
 compute_mahalanobis <- function(index, data, z) {
 
-  mt <- cov(data[z, ,drop=FALSE]) * (sum(z) - 1) / (length(z) - 2)
-  mc <- cov(data[!z, ,drop=FALSE]) * (sum(!z) - 1) / (length(!z) - 2)
-  cv <- mt + mc
+        if (!all(is.finite(data))) stop("Infinite or NA values detected in data for Mahalanobis computations.")
 
-  inv.scale.matrix <- try(solve(cv))
-
-  if (inherits(inv.scale.matrix,"try-error"))
-  {
-     dnx <- dimnames(cv)
-     s <- svd(cv)
-     nz <- (s$d > sqrt(.Machine$double.eps) * s$d[1])
-     if (!any(nz))
-       stop("covariance has rank zero")
-
-     inv.scale.matrix <- s$v[, nz] %*% (t(s$u[, nz])/s$d[nz])
-     dimnames(inv.scale.matrix) <- dnx[2:1]
-  }
-
-
-  sqrt(apply(index, 1, function(pair) {
-    t(as.matrix(data[pair[1],] - data[pair[2],])) %*% inv.scale.matrix %*% as.matrix(data[pair[1],] - data[pair[2],])
-  }))
+	mt <- cov(data[z, ,drop=FALSE]) * (sum(z) - 1) / (length(z) - 2)
+	mc <- cov(data[!z, ,drop=FALSE]) * (sum(!z) - 1) / (length(!z) - 2)
+	cv <- mt + mc
+	
+	inv.scale.matrix <- try(solve(cv))
+	
+	if (inherits(inv.scale.matrix,"try-error"))
+	{
+		dnx <- dimnames(cv)
+		s <- svd(cv)
+		nz <- (s$d > sqrt(.Machine$double.eps) * s$d[1])
+		if (!any(nz))
+		stop("covariance has rank zero")
+		
+		inv.scale.matrix <- s$v[, nz] %*% (t(s$u[, nz])/s$d[nz])
+		dimnames(inv.scale.matrix) <- dnx[2:1]
+	}
+	
+	nv <- nrow(index)
+	
+	result <- .C('mahalanobisHelper',
+    as.integer(nv),
+    as.integer(ncol(data)),
+    t(data[index[, 1], ]),
+    t(data[index[, 2], ]),
+    t(inv.scale.matrix),
+    result=numeric(nv), PACKAGE='optmatch')$result
+	
+	return(result)
 }
 
 # short alias if we need it
@@ -172,31 +209,49 @@ compute_mahal <- compute_mahalanobis
 
 compute_euclidean <- function(index, data, z) {
 
+  if (!all(is.finite(data))) stop("Infinite or NA values detected in data for distance computations.")
   sqrt(apply(index, 1, function(pair) {
-    t(as.matrix(data[pair[1],] - data[pair[2],])) %*% as.matrix(data[pair[1],] - data[pair[2],])
-  }))
 
+    pair.diff <- as.matrix(data[pair[1],] - data[pair[2],])
+    t(pair.diff) %*% pair.diff
+  
+  }))
 }
 
 # short alias
 compute_euclid <- compute_euclidean
 
-#' @details The \code{glm} method accepts a fitted propensity
-#' model, extracts distances on the linear propensity score (logits of
-#' the estimated conditional probabilities), and rescales the distances
-#' by the reciprocal of the pooled s.d. of treatment- and control-group propensity scores.
-#' (The scaling uses \code{mad}, for resistance to outliers, by default; this can be
+#' @details The \code{glm} method assumes its first argument to be a fitted propensity
+#' model. From this it extracts distances on the \emph{linear} propensity score: fitted values
+#' of the linear predictor, the link function applied to the estimated conditional probabilities, 
+#' as opposed to the estimated conditional probabilities themselves (Rosenbaum \& Rubin, 1985).  
+#' For example, a logistic model (\code{glm} with \code{family=binomial()}) has the logit function 
+#' as its link, so from such models \code{match_on} computes distances in terms of logits of
+#' the estimated conditional probabilities, i.e. the estimated log odds. 
+#' 
+#' Optionally these distances are also rescaled. The default is to rescale, by the reciprocal of 
+#' an outlier-resistant variant of the pooled s.d. of propensity scores.
+#' (Outlier resistance is obtained by the application of \code{mad}, as opposed to \code{sd}, 
+#' to linear propensity scores in the treatment; this can be
 #' changed to the actual s.d., or rescaling can be skipped entirely, by
 #' setting argument \code{standardization.scale} to \code{sd} or \code{NULL}, respectively.) 
-#' The resulting distance matrix is the absolute difference between treated and
-#' control units on the rescaled propensity scores. This method relies on the
-#' \code{numeric} method, so you may pass a \code{caliper} argument.
+#' The overall result records absolute differences between treated and
+#' control units on linear, possibly rescaled, propensity scores. 
+#' 
+#' In addition, one can impose a caliper in terms of these distances by providing a scalar as a 
+#' \code{caliper} argument, forbidding matches between treatment and control units differing in the 
+#' calculated propensity score by more than the specified caliper.  For example, Rosenbaum and Rubin's (1985) 
+#' caliper of one-fifth of a pooled propensity score s.d. would be imposed by specifying \code{caliper=.2}, 
+#' in tandem either with the default rescaling or, to follow their example even more closely, with the 
+#' additional specification \code{standardization.scale=sd}. Propensity calipers are beneficial 
+#' computationally as well as statistically, for reasons indicated in the below discussion of 
+#' the \code{numeric} method.
 #'
 #' @param standardization.scale Standardizes the data based on the median absolute deviation (by default).
-#' @usage \S4method{match_on}{glm}(x, within = NULL, standardization.scale = mad, ...)
+#' @usage \S4method{match_on}{glm}(x, within = NULL, caliper = NULL, standardization.scale = mad, ...)
 #' @rdname match_on-methods
 #' @aliases match_on,glm-method
-setMethod("match_on", "glm", function(x, within = NULL, standardization.scale = mad, ...)
+setMethod("match_on", "glm", function(x, within = NULL, caliper = NULL, standardization.scale = mad, ...)
 {
   stopifnot(all(c('y', 'linear.predictors','data') %in% names(x)))
   z <- x$y > 0
@@ -208,7 +263,7 @@ setMethod("match_on", "glm", function(x, within = NULL, standardization.scale =
 
   lp.adj <- x$linear.predictors/pooled.sd
 
-  match_on(lp.adj, within = within, z = z, ...)
+  match_on(lp.adj, within = within, caliper = caliper, z = z, ...)
 })
 
 szn.scale <- function(x, Tx, standardizer = mad, ...) {
@@ -221,10 +276,11 @@ szn.scale <- function(x, Tx, standardizer = mad, ...) {
 #' the \code{bigglm} function from package \sQuote{biglm}, which can
 #' handle bigger data sets than the ordinary glm function can.
 #'
-#' @usage \S4method{match_on}{bigglm}(x, within = NULL, data = NULL, standardization.scale = mad, ...)
+#' @usage \S4method{match_on}{bigglm}(x, within = NULL, caliper = NULL, data =
+#' NULL, standardization.scale = mad, ...)
 #' @rdname match_on-methods
 #' @aliases match_on,bigglm-method
-setMethod("match_on", "bigglm", function(x, within = NULL, data = NULL, standardization.scale = mad, ...)
+setMethod("match_on", "bigglm", function(x, within = NULL, caliper = NULL, data = NULL, standardization.scale = mad, ...)
 {
   if (is.null(data)) {
     stop("data argument is required for computing match_ons from bigglms")
@@ -256,18 +312,25 @@ are there missing values in data?")
   theps <- as.vector(theps / pooled.sd)
   names(theps) <- rownames(data)
 
-  match_on(theps, within = within, z = z, ... )    
+  match_on(theps, within = within, caliper = caliper, z = z, ... )    
 })
 
-#' @details The \code{numeric} method returns the absolute difference for treated and control units computed using
-#' the vector of scores \code{x}. Either \code{x} or \code{z} must have names.
-#' @param caliper The width of a caliper to fit on the difference of scores.
-#'   This can improve efficiency versus first creating all the differences and
-#'   then filtering out those entries that are larger than the caliper.
-#' @usage \S4method{match_on}{numeric}(x, within = NULL, z, caliper = NULL, ...)
+
+# Note that Details for the glm method, above, refers to the below for discussion of computational
+# benefits of calipers -- if that's changed here, adjust there accordingly. 
+#' @details The \code{numeric} method returns absolute differences between treated and control units'
+#' values of \code{x}. If a caliper is specified, pairings with \code{x}-differences greater than it 
+#' are forbidden.  Conceptually, those distances are set to \code{Inf}; computationally, if either of 
+#' \code{caliper} and \code{within} has been specified then only information about permissible pairings 
+#' will be stored, so the forbidden pairings are simply omitted. Providing a \code{caliper} argument here, 
+#' as opposed to omitting it and afterwards applying the \code{\link{caliper}} function, reduces 
+#' storage requirements and may otherwise improve performance, particularly in larger problems.
+#' 
+#' For the numeric method, \code{x} must have names.
+#' @usage \S4method{match_on}{numeric}(x, within = NULL, caliper = NULL, z, ...)
 #' @rdname match_on-methods
 #' @aliases match_on,numeric-method
-setMethod("match_on", "numeric", function(x, within = NULL, z, caliper = NULL, ...) {
+setMethod("match_on", "numeric", function(x, within = NULL, caliper = NULL, z, ...) {
 
   if(missing(z) || is.null(z)) {
     stop("You must supply a treatment indicator, 'z', when using the numeric match_on method.")
@@ -280,6 +343,10 @@ setMethod("match_on", "numeric", function(x, within = NULL, z, caliper = NULL, .
   z <- toZ(z)
 
   if(!is.null(caliper)) {
+    if (length(caliper) > 1) {
+      stop("Argument `caliper` must be a scalar value, not a vector.")
+    }
+
     allowed <- scoreCaliper(x, z, caliper)
     
     if (!is.null(within)) {
@@ -289,7 +356,7 @@ setMethod("match_on", "numeric", function(x, within = NULL, z, caliper = NULL, .
     }
   }
   
-  f <- function(index, data, z) { abs(apply(index, 1, function(pair) { data[pair[1]] - data[pair[2]] })) }
+  f <- function(index, data, z) { abs(data[index[,1]] - data[index[,2]]) }
 
   makedist(z, x, f, within)
 })
@@ -339,13 +406,17 @@ scoreCaliper <- function(x, z, caliper) {
 #' 
 #' @rdname match_on-methods
 #' @aliases match_on,InfinitySparseMatrix-method
-setMethod("match_on", "InfinitySparseMatrix", function(x, within = NULL, ...) {
-  return(x)
+setMethod("match_on", "InfinitySparseMatrix", function(x, within = NULL, caliper = NULL, ...) {
+  if(is.null(caliper)) { return(x) }   
+
+  return(x + optmatch::caliper(x, width = caliper))
 }) # just return the argument
 
 #' @rdname match_on-methods
 #' @aliases match_on,matrix-method
-setMethod("match_on", "matrix", function(x, within = NULL, ...) {
-  return(x)
+setMethod("match_on", "matrix", function(x, within = NULL, caliper = NULL, ...) {
+  if(is.null(caliper)) { return(x) }   
+
+  return(x + optmatch::caliper(x, width = caliper))
 }) # just return the argument
 
