@@ -85,7 +85,7 @@ summary.optmatch <- function(object,
       stop("'summary' method unable to recreate data. Consider passing 'data' argument to 'pairmatch' or 'fullmatch'.")  
     }
 
-    so$balance <- xBalance(fmla = formula(propensity.model),
+    so$balance <- RItools::xBalance(fmla = formula(propensity.model),
                            strata = strata,
                            data = data,
                            report = c('adj.means', 'z.scores', 'chisquare.test'),
